"""
Raindrop.io MCP Server.

A FastMCP-based Model Context Protocol server for Raindrop.io APIs,
providing AI agents with seamless access to bookmark management functions.
"""

__version__ = "0.1.0"
__author__ = "Raindrop.io MCP Team"
__description__ = "MCP Server for the Raindrop.io API"

__all__ = [
    "__version__",
    "__author__",
    "__description__",
]
